package clarion.acs.net;
import clarion.acs.net.auxiliary.*;
import clarion.system.*;
import clarion.common.*;
import clarion.nacs.*;
import clarion.acs.gs.*;
import clarion.acs.wm.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to implement a Fixed Rule. It extends the more generic rule class
 * <em>TLRule</em>.
 * <li> summary of the important methods. <br>
 * <ol>
 * <li> public boolean decideAction( short[] state, short[] exState, short[] gsState,
 * short[] wmState ): Decides an action by this FR rule.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model <br>
 * This class implements the Fixed Rule which is one of the three rule knowledge
 * types in the top level of an ACS action-decision network.
 * <li> Usage of this class. <br>
 * It is instantiated inside an <em>TLRuleSet</em> object.
 * @author Xi Zhang
 *
 */

public class FixRule extends TLRule {

   /** Decide which FR subset is used. */
   protected int groupId;
   /** suggested action is pre-endcoded or not. */
   protected boolean actionAvailFlag;

   /** constructor.
    *  @param groupIdx the index to the group the associated agent is in.
    *  @param netIdx the network index.
    *  @param setIdx the index to FR rule list.
    *  @param global the associated global settings. */
   public FixRule( int subsysIdx, int groupIdx, int netIdx, int setIdx, Global global )
   {
       super( subsysIdx, netIdx, Global.FR, setIdx, global );
       this.groupId = groupIdx;

       //System.out.println("groupIdx = " + groupId );

       suggestedAction = new short[ 2 ][][];
       conditions = taskClarion.getFrCondition( groupId, netIdx, setIdx );
       suggestedAction = taskClarion.getFrAction( groupId, netIdx, setIdx );

       //System.out.println( "action = " + suggestedAction[0] );

       if( suggestedAction == null ){
           actionAvailFlag = false;
           suggestedAction = new short[ 2 ][][];
           suggestedAction[Global.ACTION_NORMAL] = new short[ OUTPUT_DIM_NUM ][];
           if( subsysIdx == Global.ACS && global.isNacsOn() )
               suggestedAction[Global.ACTION_CONTROL] = new short[ CONTROL_OUTPUT_DIM_NUM ][];
       }
       else
           actionAvailFlag = true;
   }

   /** Sets the group ID.
    *  @param id the group ID to set. */
   public void setGroupId( int id )
   {
      groupId = id;
   }

   /** Decides the action.
    *  @param state curent state.
    *  @param exState external state.
    *  @param gsState goal structure state.
    *  @param wmState working memory state.
    *  @return <tt>true</tt> if this rule is matched.
    *  <tt>false>, otherwise. */
   public boolean decideAction( short[] state, short[] exState, short[] gsState, short[] wmState )
   {
       boolean matchFlag = false;
       if( conditions != null )
           matchFlag = super.decideAction( state, exState, gsState, wmState );

       //if( matchFlag )
        //   System.out.println( "the match fix rule = " + this );

       if( conditions == null || actionAvailFlag == false )
       {
           //System.out.println( global.getClarion().getSensoryInput() );
           //System.out.println( global.getGS().getActiveItem() );
           //System.out.println( global.getWM().getItems() );

           GoalChunk goalItem = global.getGsOption()!=Global.GS_NON ? global.getGS().getActiveItem():null;
           Chunk[] wmItems = global.isWmOn() ? global.getWM().getItems():null;

           /*
           System.out.println( "before calling " );
           System.out.println( "suggested action address : " + suggestedAction);
           for( int  i = 0; i < suggestedAction.length; i++ ){
                System.out.println( "suggested action address 1-dim (i): " + suggestedAction[i] );
                for( int  j = 0; j < suggestedAction[i].length; j++ )
                     System.out.println( "suggested action address 2-dim (i,j): " + suggestedAction[i][j] );
           }
           */

           ruleSupport = taskClarion.decideFrAction( groupId, netIdx, setIdx,
                         global.getClarion().getSensoryInput(), goalItem, wmItems, suggestedAction );

           /*
           System.out.println( "after calling " );
           System.out.println( "suggested action address : " + suggestedAction);
           for( int  i = 0; i < suggestedAction.length; i++ ){
                System.out.println( "suggested action address 1-dim (i): " + suggestedAction[i] );
                for( int  j = 0; j < suggestedAction[i].length; j++ )
                     System.out.println( "suggested action address 2-dim : (i,j)" + suggestedAction[i][j] );
           }
           */


           if( Math.abs(ruleSupport-1.0) < 0.01 )
               matchFlag = true;
       }

       return matchFlag;
   }

/*************************** OVERRIED METHODS *********************************/
  /** methods from class <em>Object</em> */
  public String toString()
  {
       return "net = " + netIdx + ", fix rule: " + setIdx + "\n";
  }
}