package clarion.options;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import clarion.options.*;
import clarion.options.transfer.*;
import clarion.system.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to provide a frame to collect I/O and goal dimension information
*  from user's task. It extends the <tt>selectionTemplate</tt> class.
 * <ol>
 * <li> public void initGui(): initialize the Graphical components in this frame.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model <br>
 * This class is part of GUI embedded in one selection frame.
 * <li> Usage of this class. <br>
 * It is instantiated inside a <em>SelectionTemplate</em> object.
 * </ul>
 * @author Xi Zhang
 *
 */

public class OptionDims extends SelectionTemplate{

  int dimNum, maxDimNum, featType;
  String capName;
  JLabel capLabel = new JLabel();
  JPanel panel = new JPanel();
  JLabel noteLabel1 = new JLabel();
  JLabel noteLabel2 = new JLabel();
  JLabel dimNumLabel = new JLabel();
  JTextField dimNumField = new JTextField();
  JLabel dimLabel = new JLabel();
  JLabel dimIdxLabel = new JLabel();
  JLabel dimNameLabel = new JLabel();
  JLabel[] dimLabels;
  JTextField[] dimIdxFields;
  JTextField[] dimNameFields;
  JButton returnButton = new JButton();
  // the dim option
  Option option;

  /** constructor.
   *  @param superFrame the frame this frame is embedded in.
   *  @param width the width of this frame.
   *  @param height the height of this frame.
   *  @param name   the caption.
   *  @param maxDimNum the maximum number of dimensions this frame allow to
   *  collect informaiton from.
   *  @param manager the transfter manager in charge of data transfer.
   *  @param featType the feature type: INPUT, OUTPUT, or Goal feature.
   *  @param option the option this frame is associated with. */
  public OptionDims( JFrame superFrame, int width, int height, String name, int maxDimNum,
                    TransferManager manager,  int featType, Option option )
  {
    super( superFrame, width, height, manager );
    capName = name;
    this.maxDimNum = maxDimNum;
    this.featType = featType;
    this.option = option;
    dimLabels = new JLabel[maxDimNum];
    dimIdxFields = new JTextField[maxDimNum];
    dimNameFields = new JTextField[maxDimNum];
    for( int i = 0; i < maxDimNum; i++ ){
         dimLabels[i] = new JLabel();
         dimIdxFields[i] = new JTextField();
         dimNameFields[i] = new JTextField();
    }
    initGui();
  }

  /** initialize the Graphical components in this frame. */
  public void initGui()
  {
    if( parent != null )
        setSize( 440, 720 );
    else
        setSize( width, height );
    Border BORDER = BorderFactory.createLineBorder(Color.black);
    Color COLOR = SystemColor.activeCaption;
    String[] onOffTexts = { "ON", "OFF" };
    // caption.
    initLabel( capLabel, capName, new Font("Dialog",1,14), SwingConstants.CENTER,
               new Rectangle((width-300)/2, 2, 300, 25) );
    // panel.
    initPanel( panel, BORDER, null, new Rectangle(10, 30, width-30, height-85) );
    dimNum = option.getOptionItem(0).getParameter(Global.DIM_SPEC_INDEX).getCount();
    initLabel( dimNumLabel, "Number of Dimensions", new Rectangle(10, 4, 170, 17));
    initField( dimNumField, String.valueOf( dimNum ), new Rectangle(180, 4, 24, 17), 3 );
    dimNumField.addActionListener(new ActionListener() {
             public void actionPerformed(ActionEvent e) {
                try{
                  int num = Integer.parseInt(dimNumField.getText().trim());
                  if( num < 1 || num > maxDimNum ){
                      JOptionPane.showMessageDialog( null, "out of range, reset !",
                                                    "Error", JOptionPane.ERROR_MESSAGE );
                      dimNumField.setText( String.valueOf(dimNum) );
                  }
                  else{
                      dimNum = num;
                      option.getOptionItem(0).getParameter(Global.DIM_SPEC_TYPE).setCount( dimNum );
                      for( int i = 0; i < dimNum; i++ )
                           option.getOptionItem(0).getParameter(Global.DIM_SPEC_TYPE).setIntValue(i, featType );
                      option.getOptionItem(0).getParameter(Global.DIM_SPEC_INDEX).setCount( dimNum );
                      for( int i = 0; i < maxDimNum; i++ ){
                           dimLabels[i].setEnabled( i < dimNum );
                           dimIdxFields[i].setEnabled( i < dimNum );
                           dimNameFields[i].setEnabled( i < dimNum );
                      }
                  }
                }
                catch( NumberFormatException e1 ){
                   JOptionPane.showMessageDialog( null, "Not a number, reset !",
                                                    "Error", JOptionPane.ERROR_MESSAGE );
                   dimNumField.setText( String.valueOf(dimNum) );
                }
             }
         });
    // note labels.
    initLabel( noteLabel1, "(Note: input the index of a dimension on the master dimension",
               new Rectangle(10,30,400,17));
    initLabel( noteLabel2, "list or the name of a dimension)", new Rectangle(10, 50, 400, 17));
    initLabel( dimLabel, "Dimensions", new Rectangle(10, 70, 170, 17));
    initLabel( dimIdxLabel, "Index to the dimension list", new Rectangle(100, 70, 180, 17) );
    initLabel( dimNameLabel, "Name", new Rectangle(260, 70, 100, 17) );
    for( int i = 0; i < maxDimNum; i++ ){
         initLabel( dimLabels[i], "dimension "+ i, new Rectangle(10, 90+20*i, 80, 17) );
         int dimIdx = i;
         if ( i < dimNum ){
              //System.out.println( "dimNum = " + dimNum );
              dimIdx = option.getOptionItem(0).getParameter(Global.DIM_SPEC_INDEX).getIntValues()[i];
         }
         initField( dimIdxFields[i], String.valueOf(dimIdx), new Rectangle(100, 90+20*i, 100, 17) );
         String name = "dim " + i;
         if( i < dimNum ){
             name = manager.getFeatureName( featType, dimIdx );
         }
         initField( dimNameFields[i], name, new Rectangle(260, 90+20*i, 100, 17) );
         if( i >= dimNum ){
             dimLabels[i].setEnabled( false );
             dimIdxFields[i].setEnabled( false );
             dimNameFields[i].setEnabled( false );
         }
         final int focus = i;
         dimIdxFields[i].addActionListener( new ActionListener() {
             public void actionPerformed(ActionEvent e) {
                try{
                  int maxDimIdx = manager.getFeatureNum( featType ) -1;
                  int idx = Integer.parseInt(dimIdxFields[focus].getText().trim());
                  if( idx < 0 || idx > maxDimIdx ){
                      JOptionPane.showMessageDialog( null, "min index = 0, max index = " + maxDimIdx + ", reset!",
                                                    "Error", JOptionPane.ERROR_MESSAGE );
                      dimIdxFields[focus].setText(
                          String.valueOf(option.getOptionItem(0).getParameter(Global.DIM_SPEC_INDEX).getIntValues()[focus]));
                  }
                  else{
                      option.getOptionItem(0).getParameter(Global.DIM_SPEC_INDEX).setIntValue( focus, idx );
                      dimNameFields[focus].setText(manager.getFeatureName(featType, idx));
                  }
                }
                catch( NumberFormatException e1 ){
                   JOptionPane.showMessageDialog( null, "Not a number, reset !",
                                                    "Error", JOptionPane.ERROR_MESSAGE );
                   dimIdxFields[focus].setText(
                      String.valueOf(option.getOptionItem(0).getParameter(Global.DIM_SPEC_INDEX).getIntValues()[focus]));
                }
             }
         });
         dimNameFields[i].addActionListener( new ActionListener() {
             public void actionPerformed(ActionEvent e) {
                String name = dimNameFields[focus].getText();
                int idx = manager.searchFeatureName( featType, name );
                if( idx < 0 ){
                    JOptionPane.showMessageDialog( null, "no such feature name!",
                                                  "Error", JOptionPane.ERROR_MESSAGE );
                    dimNameFields[focus].setText( manager.getFeatureName(featType,
                      Integer.parseInt(dimIdxFields[focus].getText())));
                }
                else{
                    option.getOptionItem(0).getParameter(Global.DIM_SPEC_INDEX).setIntValue( focus, idx );
                    dimIdxFields[focus].setText(
                      String.valueOf(option.getOptionItem(0).getParameter(Global.DIM_SPEC_INDEX).getIntValues()[focus]));
                }
             }
         });
    }
    // return button.
    returnButton.setText( "RETURN" );
    initReturnButton( returnButton, new Rectangle(width-120, height-40, 100, 20) );
    // add components.
    panel.add( noteLabel1 );
    panel.add( noteLabel2 );
    panel.add( dimLabel );
    panel.add( dimNumLabel );
    panel.add( dimNumField );
    panel.add( dimIdxLabel );
    panel.add( dimNameLabel );
    panel.add( dimNumLabel );
    for( int i = 0; i < maxDimNum; i++ ){
         panel.add( dimLabels[i] );
         panel.add( dimIdxFields[i] );
         panel.add( dimNameFields[i] );
    }
    if( parent != null ){
        parent.add(capLabel, null);
        parent.add(panel, null);
        parent.add(returnButton, null);
    }
    else{
        this.getContentPane().add(capLabel, null);
        this.getContentPane().add(panel, null);
        this.getContentPane().add(returnButton, null);
    }
    this.setEnabled( false );
    this.setVisible( false );
  }
}
