package clarion.options.acs;
import javax.swing.*;
import java.awt.*;
import clarion.system.*;
import clarion.options.*;
import clarion.options.transfer.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class extends the <tt>SelectionTemplate</tt> class and is the class
 * for configuration of options relevant to Working Memory by GUI. It contains
 * the options such as the size of working memory, threshold and working memory
 * item BLA etc.
 *
 *
 * <li> summary of the important methods <br>
 * <ol>
 * <li> public void initGui(): initialize the GUI components in this frame.
 * </ol>
 * </ul>
 * <li> Usage of this class. <br>
 * It is part of GUI of overall ACS networks configuration and it is instantiated in
 * <tt>GuiACS</tt> class.
 * </ul>
 * @author Xi Zhang
 *
 */

public class SelectionAcsWM extends SelectionTemplate{
  JLabel capLabel = new JLabel();
  JPanel panel_1 = new JPanel();
  JLabel panelLabel_1 = new JLabel();
  JLabel sizeOptLabel = new JLabel();
  JTextField sizeOptField = new JTextField();
  JLabel thOptLabel = new JLabel();
  JTextField thOptField = new JTextField();
  JLabel flagOptLabel = new JLabel();
  JLabel flagLabel = new JLabel();
  JTextField flagField = new JTextField();
  JRadioButton[] flagButtons = new JRadioButton[2]; {
    for( int i = 0; i < 2; i++ )
         flagButtons[i] = new JRadioButton();
  }
  JLabel actionByOptLabel = new JLabel();
  JRadioButton[] actionByButtons = new JRadioButton[2];{
    for( int i = 0; i < 2; i++ )
         actionByButtons[i] = new JRadioButton();
  }
  JLabel flagByOptLabel = new JLabel();
  JRadioButton[] flagByButtons = new JRadioButton[2]; {
    for( int i = 0; i < 2; i++ )
         flagByButtons[i] = new JRadioButton();
  }
  JPanel panel_2 = new JPanel();
  JButton returnButton = new JButton();

  public SelectionAcsWM( JFrame superFrame, int width, int height, TransferManager manager )
  {
    super( superFrame, width, height, manager );
    initGui();
  }

  /** initialize the GUI components in this frame. */
  public void initGui() {
    final int SIZE_WM = 0, TH_WM = 1, FLAG_WM = 2, ACT_DEC_MODE = 3;
    final int FLAG_DEC_MODE = 4, BLA_OPT = 5;
    String onOffTexts[] = { "ON", "OFF" };

    options = manager.getOptions( Global.ACS, Global.ACS_WM, 0, 0, 0, 0 );
    setSize( width, height );
    this.getContentPane().setLayout( null );

    initLabel( capLabel, "WORKING MEMORY OPTIONS", new java.awt.Font("Dialog", 1, 16),
               SwingConstants.CENTER, new Rectangle(140, 1, 308, 27));
    initPanel( panel_1, BorderFactory.createLineBorder(Color.black), null,
               new Rectangle(10, 35, 270, 275));
    initLabel( panelLabel_1, "Options and parameters for working memory",
               new Rectangle(4, 8, 280, 17), SystemColor.activeCaption );
    // size of WM.
    initLabel( sizeOptLabel, "total capacity of working memory", new Rectangle(10, 30, 190, 17));
    int sizeWM = options[SIZE_WM].getOptionItem(0).getParameter(0).getIntValue();
    initField( sizeOptField, String.valueOf(sizeWM), new Rectangle(200, 30, 24, 17), 5,
               options[SIZE_WM], 0, 0 );
    // WM threshold.
    initLabel( thOptLabel, "working memory threshold", new Rectangle(10, 55, 158, 17));
    double thWM = options[TH_WM].getOptionItem(0).getParameter(0).getDoubleValue();
    initField( thOptField, String.valueOf(thWM), new Rectangle(170, 55, 32, 17), 5,
               options[TH_WM], 0, 0 );
    // flags in WM
    Rectangle[] flagRects = {new Rectangle(10, 100, 45, 17), new Rectangle(10, 123, 47, 17)};
    initLabel( flagLabel, "number of flags", new Rectangle(65, 100, 96, 17));
    int flagNum = options[FLAG_WM].getOptionItem(0).getParameter(0).getIntValue();
    initField( flagField, String.valueOf( flagNum ), new Rectangle(160, 100, 24, 17), 5 );
    initButtonBlock( panel_1, flagOptLabel, "flags in working memory", new Rectangle(10, 80, 146, 17),
                     flagButtons, onOffTexts, flagRects, options[FLAG_WM], 0, flagField );
    // WM actions by
    String[] actionByTexts = { "top level only(using fixed rules)", "both levels" };
    Rectangle[] actionByRects = {new Rectangle(10, 165, 240, 17),new Rectangle(10, 185, 92, 17) };
    initButtonBlock( panel_1, actionByOptLabel, "working memory actions by", new Rectangle(10, 147, 164, 17),
                     actionByButtons, actionByTexts, actionByRects, options[ACT_DEC_MODE] );
    // flag actions by
    String[] flagByTexts = { "top level only(using fixed rules)", "both levels" };
    Rectangle[] flagByRects = {new Rectangle(10, 230, 240, 17),new Rectangle(10, 252, 87, 17)};
    initButtonBlock( panel_1, flagByOptLabel, "flag actions by", new Rectangle(10, 211, 92, 17),
                     flagByButtons, flagByTexts, flagByRects, options[FLAG_DEC_MODE] );
    // WM BLA.
    initPanel( panel_2, BorderFactory.createLineBorder(Color.black), null,
               new Rectangle( 290, 35, 290, 275 ) );
    this.initBlaBlock( panel_2, "working memory base-level activation options", options[BLA_OPT] );
    // return button.
    initReturnButton( returnButton, "RETURN", new Rectangle(460, 326, 100, 29));
    // add components.
    panel_1.add(panelLabel_1, null);
    panel_1.add(sizeOptLabel, null);
    panel_1.add(sizeOptField, null);
    panel_1.add(thOptLabel, null);
    panel_1.add(thOptField, null);
    //panel_1.add(flagOptLabel, null);
    panel_1.add(flagLabel, null);
    panel_1.add(flagField, null);
    //panel_1.add(actionByOptLabel, null);
    //panel_1.add(flagByOptLabel, null);
    this.getContentPane().add( capLabel, null );
    this.getContentPane().add( panel_1, null );
    this.getContentPane().add( panel_2, null );
    this.getContentPane().add( returnButton, null );
  }
}
