package clarion.options.acs.net;
import java.awt.event.*;
import javax.swing.*;
import clarion.system.*;
import clarion.options.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to show the top-level screen of configuring one generic ACS Network
 * components by GUI. It extends the base class <em>GuiTemplate</em>. It contains
 * the following configuration entries : BASICS, IDN, RER, IRL and FIXED RULES.
 *
 * <li> summary of the important methods. <br>
 * <ol>
 * <li> public void returnFrom( int idx, int command ): update the GUI components of
 * this frame only resulting from lower level.
 * <li> public void update(): update the GUI components of this frame only resulting
 * from upper level.
 * <li> public void addActionListeners( final JFrame mainFrame ): event listeners
 * for the buttons.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model <br>
 * This class is to control the process of configuring one generic ACS network
 * components by GUI (on-line configuration).
 * <li> Usage of this class. <br>
 * It is instantiated inside a <em>GuiAcsNets</em> or <em>GuiAcsExNets</em> object.
 * </ul>
 * @author Xi Zhang
 *
 */


public class GuiAcsNet extends GuiTemplate{

  final int BL = 0, RER = 1, IRL = 2, FR = 3;
  String buttonTexts[] = { "BASICS", "IDN", "RER", "IRL", "FIXED RULES" };
  SelectionTemplate basics;
  SelectionTemplate netComps[][];
  int netType, netIdx;

  public GuiAcsNet( int netType, int netIdx )
  {
     super( 5, "OPTIONS OF AN IDN AND CORRESPONDING RULE GROUP" );
     setButtonTexts( buttonTexts );
     this.netType = netType;
     this.netIdx = netIdx;
     basics = new SelectionAcsNetBasic(  this, 880, 620, netType, netIdx, manager );
     // initialize component GUI.
     netComps = new SelectionTemplate[ OPT_NUM-1 ][];
     netComps[BL] = new SelectionTemplate[ Global.MAX_RULE_NUMS[BL] ];
     if( !manager.isAcsCompOn( netType, netIdx, BL ) )
         optButtons[BL+1].setEnabled( false );
     else{
         netComps[BL][0] = new SelectionAcsIdn( this, 690, 400, netType, netIdx, manager );
     }
     netComps[RER] = new SelectionTemplate[ Global.MAX_RULE_NUMS[RER] ];
     if( !manager.isAcsCompOn( netType, netIdx, RER ) )
         optButtons[RER+1].setEnabled( false );
     else{
         netComps[RER][0] = new SelectionAcsRer( this, 1030, 800, netType, netIdx, 0, manager );
     }
     netComps[IRL] = new SelectionTemplate[ Global.MAX_RULE_NUMS[IRL] ];
     if( !manager.isAcsCompOn( netType, netIdx, IRL ) )
         optButtons[IRL+1].setEnabled( false );
     else{
         int irlSetNum = manager.getAcsCompFileNum( netType, netIdx, IRL );
         for( int i = 0; i < irlSetNum; i++ )
              netComps[IRL][i] = new SelectionAcsIrl( this, 1030, 800, netType, netIdx, i, manager );
     }
     netComps[FR] = new SelectionTemplate[ Global.MAX_RULE_NUMS[FR] ];
     if( !manager.isAcsCompOn( netType, netIdx, FR ) )
         optButtons[FR+1].setEnabled( false );
     else{
         int frRuleNum = manager.getAcsCompFileNum( netType, netIdx, FR );
         for( int i = 0; i < frRuleNum; i++ )
              netComps[FR][i] = new SelectionAcsFR(  this, 980, 600, netType, netIdx, i, manager );
     }
  }

  /** update the GUI components of this frame only resulting from lower level. */
  public void returnFrom( int idx, int command )
  {
     switch( activeButtonIdx ){
      case 0:
           setEnabled( true );
           setVisible( true );
           for( int i = 0; i < 4; i++ )
              if( manager.isAcsCompOn( netType, netIdx, i ) ){
                  optButtons[i+1].setEnabled( true );
                  int setNum = manager.getAcsCompFileNum( netType, netIdx, i );
                  for( int j = 0; j < setNum; j++ ){
                      if( netComps[i][j] == null )
                          switch(i){
                           case RER:
                                netComps[i][j] = new SelectionAcsRer( this, 1020, 800, netType, netIdx,
                                                     j, manager );
                                break;
                           case IRL:
                                netComps[i][j] = new SelectionAcsIrl( this, 1020, 800, netType, netIdx,
                                                     j, manager );
                                break;
                           case FR:
                                netComps[i][j] = new SelectionAcsFR(  this, 980, 600, netType, netIdx,
                                                     j, manager );
                                break;
                         }
                      netComps[i][j].update();
                  }
              }
              else
                  optButtons[i+1].setEnabled( false );
      case 1:
      case 2:
           setEnabled( true );
           setVisible( true );
           break;
      default:
           switch( command ){
           case 0:  // previous.
                netComps[activeButtonIdx-1][idx-1].setEnabled( true );
                netComps[activeButtonIdx-1][idx-1].setVisible( true );
                break;
           case 1: // next.
                netComps[activeButtonIdx-1][idx+1].setEnabled( true );
                netComps[activeButtonIdx-1][idx+1].setVisible( true );
                break;
           case 2: // return to top.
                setEnabled( true );
                setVisible( true );
                break;
           }
           break;
     }
  }

  /** update the GUI components of this frame only resulting from upper level. */
  public void update()
  {
     basics.update();
     netComps[BL][0].update();
  }

  /** event listeners for the buttons. */
  public void addActionListeners( final JFrame mainFrame )
  {
     for( int i = 0; i < OPT_NUM; i++ ){
          final int idx = i;
          optButtons[ idx ].addActionListener(  new  ActionListener(){
                            public void actionPerformed( ActionEvent e ){
                               activeButtonIdx = idx;
                               if( idx > 0 ){
                                   netComps[idx-1][0].setEnabled( true );
                                   netComps[idx-1][0].setVisible( true );
                               }
                               else{
                                   basics.setEnabled( true );
                                   basics.setVisible( true );
                               }
                               setEnabled( false );
                            }
                    } );
     }
  }
}
