package clarion.options.acs.net;
import java.awt.event.*;
import javax.swing.*;
import clarion.system.*;
import clarion.options.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to show the top-level screen of configuring all ACS Networks components
 * by GUI. It extends the base class <em>GuiTemplate</em>. It contains the following
 * configuration entries:  External IDNs And Corresponding Rule Groups,
 * Working Memory IDN And Corresponding Rule Group,  Goal Structure IDN And
 * Corresponding Rule Group and Options Relavant to All types of networks.
 *
 * <li> summary of the important methods. <br>
 * <ol>
 * <li> public void returnFrom(): update the GUI components when return from other
 * screens.
 * <li> public void updateGui(): update the GUI components of this frame only
 * resulting from upper level.
 * <li> public void update(): update the GUI components of this frame only resulting
 * from lower level.
 * <li> public void addActionListeners( final JFrame mainFrame ) :  event listeners
 * for the buttons.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model <br>
 * This class is to control the process of configuring all ACS networks components
*  by GUI (on-line configuration).
 * <li> Usage of this class. <br>
 * It is instantiated inside a <em>GuiACS</em> object.
 * </ul>
 * @author Xi Zhang
 *
 */

public class GuiAcsNets extends GuiTemplate{

  String buttonTexts[] = {
      "External IDNs And Corresponding Rule Groups",
      "Working Memory IDN And Corresponding Rule Group",
      "Goal Structure IDN And Corresponding Rule Group",
      "Options Relavant to All"
  };
  GuiTemplate[] networks;
  SelectionTemplate subSelection;

  public GuiAcsNets( int netTypeNum )
  {
     super( netTypeNum, "OPTIONS OF THE ACS NETWROKS" );
     setButtonTexts( buttonTexts );
     optButtons[Global.WM_NET].setEnabled( manager.getWmOption() && manager.isWmIdnOn() );
     optButtons[Global.GS_NET].setEnabled( manager.getGsOption()!= Global.GS_NON && manager.isGsIdnOn() );
     networks = new GuiTemplate[3];
     for( int i = 0; i < 3; i++ ){
          if( i == Global.EX_NET )
              networks[i] = new GuiAcsExNets( manager.getAcsExNetNum() );
          else
              networks[i] = new GuiAcsNet( i, 0 );
          networks[i].init( getFrame() );
     }
     // relevant to all nets.
     optButtons[3].setEnabled( true );
     subSelection = new SelectionAcsCor( this, 760, 330, manager );
  }

  /** update the GUI components when return from other screens. */
  public void returnFrom()
  {
     if( activeButtonIdx == 3 )  // reinforcement functions.
         update();
  }

  /** update the GUI components of this frame only resulting from upper level. */
  public void updateGui()
  {
     optButtons[Global.WM_NET].setEnabled( manager.getWmOption() && manager.isWmIdnOn() );
     optButtons[Global.GS_NET].setEnabled( manager.getGsOption()!= Global.GS_NON && manager.isGsIdnOn());
     ((GuiAcsExNets)networks[Global.EX_NET]).updateGui();
  }

  /** update the GUI components of this frame only resulting from lower level. */
  public void update()
  {
     for( int i = 0; i < OPT_NUM - 1; i++ )
          if( optButtons[i].isEnabled() ){
              if( i == Global.EX_NET )
                  ((GuiAcsExNets)networks[i]).update();
              else
                  ((GuiAcsNet)networks[i]).update();
          }
  }

  /** event listeners for the buttons. */
  public void addActionListeners( final JFrame mainFrame )
  {
     for( int i = 0; i < OPT_NUM; i++ ){
          final int idx = i;
          optButtons[ idx ].addActionListener(  new  ActionListener(){
                            public void actionPerformed( ActionEvent e ){
                               activeButtonIdx = idx;
                               if( idx < OPT_NUM-1 ){
                                   networks[idx].setEnabled( true );
                                   networks[idx].setVisible( true );
                                   setEnabled( false );
                               }
                               else{
                                   subSelection.setEnabled( true );
                                   subSelection.setVisible( true );
                                   setEnabled( false );
                               }
                            }
                    } );
     }
  }
}
