package clarion.options.acs.net;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import clarion.system.*;
import clarion.options.*;
import clarion.options.transfer.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class extends the <tt>SelectionTemplate</tt> class and is the class
 * for configuration of options relevant to the structure of an ACS network by GUI.
 * It contains the options such as selection of RER, IRL and FR rule types,
 * determinaton mode, combination mode, and three different combination types.
 *
 * <li> summary of the important methods <br>
 * <ol>
 * <li> public void initGui(): initialize the GUI components in this frame.
 * <li> public void update(): update GUI components status resulting from
 * upper level configuration.
 * </ol>
 * </ul>
 * <li> Usage of this class. <br>
 * It is part of GUI of ACS network configuration and it is instantiated in
 * <tt>GuiAcsNet</tt> class.
 * </ul>
 * @author Xi Zhang
 *
 */

public class SelectionAcsNetBasic extends SelectionTemplate{
  int netType, netIdx;
  // caption
  JLabel caption = new JLabel();
  // component
  JPanel compPanel = new JPanel();
  JLabel compLabel = new JLabel();
  JLabel rerOnLabel = new JLabel();
  JRadioButton[] rerOnButtons = new JRadioButton[ 2 ]; {
     for( int i = 0; i < 2; i++ )
          rerOnButtons[i] = new JRadioButton();
  }
  JLabel irlOnLabel = new JLabel();
  JRadioButton[] irlOnButtons = new JRadioButton[ 2 ]; {
     for( int i = 0; i < 2; i++ )
          irlOnButtons[i] = new JRadioButton();
  }
  JLabel irlNumLabel = new JLabel();
  JTextField irlNumField = new JTextField();
  JLabel frOnLabel = new JLabel();
  JRadioButton[] frOnButtons = new JRadioButton[ 2 ]; {
     for( int i = 0; i < 2; i++ )
          frOnButtons[i] = new JRadioButton();
  }
  JLabel frNumLabel = new JLabel();
  JTextField frNumField = new JTextField();
  JLabel insOnLabel = new JLabel();
  JRadioButton[] insOnButtons = new JRadioButton[ 2 ]; {
     for( int i = 0; i < 2; i++ )
          insOnButtons[i] = new JRadioButton();
  }
  JLabel ilOnLabel = new JLabel();
  JRadioButton[] ilOnButtons = new JRadioButton[ 2 ]; {
     for( int i = 0; i < 2; i++ )
          ilOnButtons[i] = new JRadioButton();
  }
  JLabel planOnLabel = new JLabel();
  JRadioButton[] planOnButtons = new JRadioButton[ 2 ]; {
     for( int i = 0; i < 2; i++ )
          planOnButtons[i] = new JRadioButton();
  }
  JLabel mcsOnLabel = new JLabel();
  JRadioButton[] mcsOnButtons = new JRadioButton[ 2 ]; {
     for( int i = 0; i < 2; i++ )
          mcsOnButtons[i] = new JRadioButton();
  }
  // eligibility
  boolean[] eligCompOns;
  JPanel eligPanel = new JPanel();
  JLabel eligLabel = new JLabel();
  JLabel eligLabel1 = new JLabel();
  JButton[] eligUDButtons = new JButton[2]; {
     eligUDButtons[0] = new JButton();
     eligUDButtons[1] = new JButton();
  }
  // cross level combination.
  boolean[] outcomeCompOns;
  boolean[][] combCompOns;{
       combCompOns = new boolean[3][];
  }
  // determination
  JPanel crossPanel = new JPanel();
  JLabel deterLabel = new JLabel();
  JRadioButton[] crossButtons = new JRadioButton[ 2 ]; {
     for( int i = 0; i < 2; i++ )
          crossButtons[i] = new JRadioButton();
  }
  // integration
  JPanel outcomePanel = new JPanel();
  JLabel outcomeLabel = new JLabel();
  JRadioButton[] outcomeButtons = new JRadioButton[ 3 ]; {
     for( int i = 0; i < 3; i++ )
          outcomeButtons[i] = new JRadioButton();
  }
  // combination methods.
  JPanel[] combPanels = new JPanel[3];{
     combPanels[0] = new JPanel();
     combPanels[1] = new JPanel();
     combPanels[2] = new JPanel();
  }
  // stochastic method
  JLabel stochLabel = new JLabel();
  JRadioButton[] stochButtons = new JRadioButton[ 2 ]; {
     for( int i = 0; i < 2; i++ )
          stochButtons[i] = new JRadioButton();
  }
  JLabel[] stochFixLabels = new JLabel[ 4 ]; {
     for( int i = 0; i < 4; i++ )
          stochFixLabels[i] = new JLabel();
  }
  JLabel[] stochVarLabels = new JLabel[ 6 ]; {
     for( int i = 0; i < 6; i++ )
          stochVarLabels[i] = new JLabel();
  }
  JTextField[][] stochFields = new JTextField[2][]; {
    stochFields[0] = new JTextField[4];
    for( int j = 0; j < 4; j++ )
         stochFields[0][j] = new JTextField();
    stochFields[1] = new JTextField[6];
    for( int j = 0; j < 6; j++ )
         stochFields[1][j] = new JTextField();
  }
  // bottom-up verification.
  JLabel buLabel = new JLabel();
  JRadioButton[] buButtons = new JRadioButton[ 3 ]; {
     for( int i = 0; i < 3; i++ )
          buButtons[i] = new JRadioButton();
  }
  JLabel[] buFixLabels = new JLabel[ 5 ]; {
     for( int i = 0; i < 5; i++ )
          buFixLabels[i] = new JLabel();
  }
  JLabel[] buVarLabels = new JLabel[ 7 ]; {
     for( int i = 0; i < 7; i++ )
          buVarLabels[i] = new JLabel();
  }
  JTextField[][] buFields = new JTextField[2][]; {
     buFields[0] = new JTextField[ 5 ];
     for( int i = 0; i < 5; i++ )
          buFields[0][i] = new JTextField();
     buFields[1] = new JTextField[ 7 ];
     for( int i = 0; i < 7; i++ )
          buFields[1][i] = new JTextField();
  }
  JButton[] buUDButtons = new JButton[2]; {
     buUDButtons[0] = new JButton();
     buUDButtons[1] = new JButton();
  }
  JLabel buFileLabel = new JLabel();
  JLabel buMethodLabel = new JLabel();
  // top-down-guidance.
  JLabel tgLabel = new JLabel();
  JRadioButton[] tgButtons = new JRadioButton[ 3 ]; {
     for( int i = 0; i < 3; i++ )
          tgButtons[i] = new JRadioButton();
  }
  JLabel[] tgFixLabels = new JLabel[ 5 ]; {
     for( int i = 0; i < 5; i++ )
          tgFixLabels[i] = new JLabel();
  }
  JLabel[] tgVarLabels = new JLabel[ 7 ]; {
     for( int i = 0; i < 7; i++ )
          tgVarLabels[i] = new JLabel();
  }
  JTextField[][] tgFields = new JTextField[2][]; {
     tgFields[0] = new JTextField[ 5 ];
     for( int i = 0; i < 5; i++ )
          tgFields[0][i] = new JTextField();
     tgFields[1] = new JTextField[ 7 ];
     for( int i = 0; i < 7; i++ )
          tgFields[1][i] = new JTextField();
  }
  JButton[] tgUDButtons = new JButton[2]; {
     tgUDButtons[0] = new JButton();
     tgUDButtons[1] = new JButton();
  }
  // return buttons.
  JButton returnButton = new JButton();
  // eiligibility option
  Option eligOption;

  String stochFixLabelTexts[] = { "p_bl =", "p_rer =", "p_irl =", "p_fr =" };
  String wsFixLabelTexts[] = { "w_bl =", "w_rer =", "w_irl =", "w_fr =",
                               "temperature for overall action seleciton = " };
  String stochVarLabelTexts[] = { "b_bl =", "b_rer =", "b_irl =", "b_fr =", "c3 =", "c4 =" };
  String wsVarLabelTexts[] = { "b_bl =", "b_rer =", "b_irl =", "b_fr =", "c3 =", "c4 =",
                               "temperature for overall action seleciton = " };

  public SelectionAcsNetBasic( JFrame superFrame, int width, int height, int netType, int netIdx,
                               TransferManager manager )
  {
    super( superFrame, width, height, manager );
    this.netType = netType;
    this.netIdx = netIdx;
    initGui();
  }

  /** initialize the GUI components in this frame. */
  public void initGui()
  {
    final int IDN_ON = 0, RER_ON = 1, IRL_ON = 2, FR_ON = 3;
    final int EX_INSTR = 4, IM_LEARN = 5, PLAN = 6, MCS = 7;
    final int DETER = 8, COMB = 9, STOCH = 10, BU = 11, TG = 12;
    final int ELIG = 13;
    final int ON = 0, OFF = 1;
    final String[] capPostfixs = {"  ( EXTERNAL )", "  ( WORKING MEMORY )", "  ( GOAL SETTING )"};
    final short[][] locations = {{26, 300, 350, 400, 200}, {26, 220, 290, 360 }};
    int type = (netType == Global.EX_NET )?0:1;

    options = manager.getOptions( Global.ACS, Global.ACS_NET, netType, netIdx, Global.ACS_BASIC, 0 );
    Border BORDER = BorderFactory.createLineBorder(Color.black);
    Color COLOR = SystemColor.activeCaption;
    String[] onOffTexts = { "ON", "OFF" };
    setSize( width, height );

    this.getContentPane().setLayout(null);
    // caption
    initLabel( caption, "BASIC OPTIONS FOR AN IDN AND ITS CORRESPONDING RULE GROUP" + capPostfixs[netType],
         new Font("Dialog",1,14), SwingConstants.CENTER, new Rectangle(80,0,(width-160),26));
    // components
    initPanel( compPanel, BORDER, null, new Rectangle(10, locations[type][0], 440, 170) );
    initLabel( compLabel, "Options for rule learning", new Rectangle(4, 6, 300, 17) );
    Rectangle[] rerOnRects = {new Rectangle(50, 28, 50, 17), new Rectangle(100, 28, 50, 17)};
    initButtonBlock( compPanel, rerOnLabel, "RER", new Rectangle(16, 28, 30, 17),
                     rerOnButtons, onOffTexts, rerOnRects, options[RER_ON] );
    Rectangle[] irlOnRects = {new Rectangle(50, 48, 50, 17), new Rectangle(100, 48, 50, 17)};
    initButtonBlock( compPanel, irlOnLabel, "IRL", new Rectangle(16, 48, 30, 17),
                     irlOnButtons, onOffTexts, irlOnRects, options[IRL_ON], 0, irlNumField );
    initLabel( irlNumLabel, "number of IRL rule subset", new Rectangle(180, 48, 160, 17) );
    initField( irlNumField, String.valueOf(options[IRL_ON].getOptionItem(0).getParameter(0).getIntValue()),
               new Rectangle(340, 48, 32, 17), 3, options[IRL_ON], 0, 0 );
    Rectangle[] frOnRects = {new Rectangle(90, 68, 60, 17), new Rectangle(150, 68, 60, 17)};
    initButtonBlock( compPanel, frOnLabel, "Fixed Rules", new Rectangle(16, 68, 70, 17),
                     frOnButtons, onOffTexts, frOnRects, options[FR_ON], 0, frNumField  );
    initLabel( frNumLabel, "number of rules", new Rectangle(230, 68, 100, 17) );
    initField( frNumField, String.valueOf(options[FR_ON].getOptionItem(0).getParameter(0).getIntValue()),
               new Rectangle(330, 68, 32, 17), 3, options[FR_ON], 0, 0 );
    Rectangle[] insOnRects = {new Rectangle(150, 88, 60, 17), new Rectangle(210, 88, 60, 17)};
    initButtonBlock( compPanel, insOnLabel, "external instructions", new Rectangle(16, 88, 120, 17),
                     insOnButtons, onOffTexts, insOnRects, options[EX_INSTR] );
    Rectangle[] ilOnRects = {new Rectangle(140, 108, 60, 17), new Rectangle(200, 108, 60, 17)};
    initButtonBlock( compPanel, ilOnLabel, "imitative learning", new Rectangle(16, 108, 110, 17),
                     ilOnButtons, onOffTexts, ilOnRects, options[IM_LEARN] );
    Rectangle[] planOnRects = {new Rectangle(130, 128, 60, 17), new Rectangle(190, 128, 60, 17)};
    initButtonBlock( compPanel, planOnLabel, "plan extraction", new Rectangle(16, 128, 110, 17),
                     planOnButtons, onOffTexts, planOnRects, options[PLAN] );
    Rectangle[] mcsOnRects = {new Rectangle(140, 148, 60, 17), new Rectangle(200, 148, 60, 17)};
    initButtonBlock( compPanel, mcsOnLabel, "determined by MCS", new Rectangle(16, 148, 110, 17),
                     mcsOnButtons, onOffTexts, mcsOnRects, options[MCS] );
    // eligibility
    if( type == 0 ){
        initPanel( eligPanel, BORDER, null, new Rectangle(10, locations[type][4], 440, 96));
        initLabel( eligLabel, "Eligibility condition of external action network", new Rectangle(4,4,280,17) );
        initLabel( eligLabel1, "( user defined )",  new Rectangle(16,28,280,17) );
        Rectangle[] eligUDRects = { new Rectangle(16, 48, 160, 20), new Rectangle(190, 48, 160, 20) };
        String[] eligUDTexts = { TaskClarion.getSample( TaskClarion.ACS, TaskClarion.ELIG ),
                               TaskClarion.getExplanation( TaskClarion.ACS, TaskClarion.ELIG ) };
        initUserDefinedBlock( eligPanel, eligUDButtons, eligUDRects, eligUDTexts );
        eligPanel.add(eligLabel, null);
        eligPanel.add(eligLabel1, null);
        eligCompOns = new boolean[eligPanel.getComponentCount()];
        eligOption = manager.getOptions( Global.ACS, Global.ACS_NET, Global.ACS_COR, 0, 0, 0, 0 );
        this.initPanelStatus( eligOption, 1, eligPanel, eligCompOns );
    }
    // stochastic combination.
    initPanel( combPanels[0], BORDER, null, new Rectangle(10, locations[type][3], 440, 170) );
    String[] stochButtonTexts = {"fixed   (each number should be > 0, the sum should be = 1)","variable"};
    Rectangle[] stochButtonRects = {new Rectangle(8, 108, 390, 17),new Rectangle(8, 28, 80, 17)};
    initButtonBlock( combPanels[0], stochLabel, "Stochastic selection parameters", new Rectangle(4, 6, 240, 17),
                     stochButtons, stochButtonTexts, stochButtonRects, options[STOCH], stochFields );
    // p_bl, p_rer, p_irl, p_fr
    Rectangle[] stochFixLabelRects = { new Rectangle(24, 128, 40, 17),new Rectangle(114, 128, 44, 17),
                                       new Rectangle(204, 128, 40, 17), new Rectangle(294, 128, 40, 17) };
    String[] stochFixFieldTexts = {
             String.valueOf(options[STOCH].getOptionItem(0).getParameter(0).getDoubleValue()),
             String.valueOf(options[STOCH].getOptionItem(0).getParameter(1).getDoubleValue()),
             String.valueOf(options[STOCH].getOptionItem(0).getParameter(2).getDoubleValue()),
             String.valueOf(options[STOCH].getOptionItem(0).getParameter(3).getDoubleValue()) };
    Rectangle[] stochFixFieldRects = { new Rectangle(64, 128, 32, 17), new Rectangle(158, 128, 32, 17),
                                       new Rectangle(244, 128, 32, 17), new Rectangle(334, 128, 32, 17)};
    initFieldBlock( combPanels[0], stochFixLabels, stochFixLabelTexts, stochFixLabelRects,
                    stochFields[0], stochFixFieldTexts, stochFixFieldRects, options[STOCH], 0 );
    // b_bl, b_rer, b_irl, b_fr, c3, c4
    Rectangle[] stochVarLabelRects = { new Rectangle(24, 48, 40, 17),new Rectangle(114, 48, 44, 17),
                                       new Rectangle(204, 48, 40, 17), new Rectangle(294, 48, 40, 17),
                                       new Rectangle(24,  68, 40, 17), new Rectangle(114, 68, 40, 17)
                                     };
    String[] stochVarFieldTexts = {
             String.valueOf(options[STOCH].getOptionItem(1).getParameter(0).getDoubleValue()),
             String.valueOf(options[STOCH].getOptionItem(1).getParameter(1).getDoubleValue()),
             String.valueOf(options[STOCH].getOptionItem(1).getParameter(2).getDoubleValue()),
             String.valueOf(options[STOCH].getOptionItem(1).getParameter(3).getDoubleValue()),
             String.valueOf(options[STOCH].getOptionItem(1).getParameter(4).getDoubleValue()),
             String.valueOf(options[STOCH].getOptionItem(1).getParameter(5).getDoubleValue())
    };
    Rectangle[] stochVarFieldRects = { new Rectangle(64, 48, 32, 17), new Rectangle(158, 48, 32, 17),
                                       new Rectangle(244, 48, 32, 17), new Rectangle(334, 48, 32, 17),
                                       new Rectangle(64,  68, 32, 17), new Rectangle(158, 68, 32, 17)
                                     };
    initFieldBlock( combPanels[0], stochVarLabels, stochVarLabelTexts, stochVarLabelRects,
                    stochFields[1], stochVarFieldTexts, stochVarFieldRects, options[STOCH], 1 );
    combCompOns[0] = new boolean[combPanels[0].getComponentCount()];
    // bottom-up verificaiton.
    initPanel( combPanels[1], BORDER, null, new Rectangle(460, 26, 400, 250) );
    String[] buButtonTexts = { "fixed weighted sum", "variable weighted sum", "correction" };
    Rectangle[] buButtonRects = { new Rectangle(8, 118, 160, 17), new Rectangle(8, 28, 160, 17),
                                  new Rectangle(8, 188, 160, 17) };
    initButtonBlock( combPanels[1], buLabel, "Bottom-up verification selection", new Rectangle(4, 6, 240, 17),
       buButtons, buButtonTexts, buButtonRects, options[BU], buFields, 2, buUDButtons );
       // fixed weighted sum.
    Rectangle[] buFixLabelRects = { new Rectangle(24, 138, 40, 17), new Rectangle(114, 138, 44, 17),
                                    new Rectangle(204, 138, 40, 17), new Rectangle(294, 138, 40, 17),
                                    new Rectangle(24, 158, 240, 17) };
    String[] buFixFieldTexts = {
             String.valueOf(options[BU].getOptionItem(0).getParameter(0).getDoubleValue()),
             String.valueOf(options[BU].getOptionItem(0).getParameter(1).getDoubleValue()),
             String.valueOf(options[BU].getOptionItem(0).getParameter(2).getDoubleValue()),
             String.valueOf(options[BU].getOptionItem(0).getParameter(3).getDoubleValue()),
             String.valueOf(options[BU].getOptionItem(0).getParameter(4).getDoubleValue())
          };
    Rectangle[] buFixFieldRects = { new Rectangle(64, 138, 32, 17), new Rectangle(158, 138, 32, 17),
                                    new Rectangle(244, 138, 32, 17), new Rectangle(334, 138, 32, 17),
                                    new Rectangle(264, 158, 32, 17) };
    initFieldBlock( combPanels[1], buFixLabels, wsFixLabelTexts, buFixLabelRects,
                    buFields[0], buFixFieldTexts, buFixFieldRects, options[BU], 0 );
      // variable weighted sum.
    Rectangle[] buVarLabelRects = { new Rectangle(24, 48, 40, 17),new Rectangle(114, 48, 44, 17),
                                    new Rectangle(204, 48, 40, 17), new Rectangle(294, 48, 40, 17),
                                    new Rectangle(24,  68, 40, 17), new Rectangle(114, 68, 40, 17),
                                    new Rectangle(24, 88, 240, 17)
                                  };
    String[] buVarFieldTexts = {
                String.valueOf(options[BU].getOptionItem(1).getParameter(0).getDoubleValue()),
                String.valueOf(options[BU].getOptionItem(1).getParameter(1).getDoubleValue()),
                String.valueOf(options[BU].getOptionItem(1).getParameter(2).getDoubleValue()),
                String.valueOf(options[BU].getOptionItem(1).getParameter(3).getDoubleValue()),
                String.valueOf(options[BU].getOptionItem(1).getParameter(4).getDoubleValue()),
                String.valueOf(options[BU].getOptionItem(1).getParameter(5).getDoubleValue()),
                String.valueOf(options[BU].getOptionItem(1).getParameter(6).getDoubleValue())
             };
    Rectangle[] buVarFieldRects = { new Rectangle(64, 48, 32, 17), new Rectangle(158, 48, 32, 17),
                                    new Rectangle(244, 48, 32, 17), new Rectangle(334, 48, 32, 17),
                                    new Rectangle(64,  68, 32, 17), new Rectangle(158, 68, 32, 17),
                                    new Rectangle(264, 88, 32, 17)
                                  };
    initFieldBlock( combPanels[1], buVarLabels, wsVarLabelTexts, buVarLabelRects,
                    buFields[1], buVarFieldTexts, buVarFieldRects, options[BU], 1 );
    // user defined.
    Rectangle[] buUDRects = { new Rectangle(24, 208, 160, 17), new Rectangle(24, 228, 160, 17) };
    String[] buUDTexts = { TaskClarion.getSample( TaskClarion.ACS, TaskClarion.BUR ),
                         TaskClarion.getExplanation( TaskClarion.ACS, TaskClarion.BUR ) };
    initUserDefinedBlock( combPanels[1], buUDButtons, buUDRects, buUDTexts );
    combCompOns[1] = new boolean[combPanels[1].getComponentCount()];
    // top-down guidance.
    initPanel( combPanels[2], BORDER, null, new Rectangle(460, 280, 400, 250) );
    String[] tgButtonTexts = { "fixed weighted sum", "variable weighted sum", "correction" };
    Rectangle[] tgButtonRects = { new Rectangle(8, 118, 160, 17), new Rectangle(8, 28, 160, 17),
                                  new Rectangle(8, 188, 160, 17) };
    initButtonBlock( combPanels[2], tgLabel, "Top-down guidance selection", new Rectangle(4, 6, 200, 17),
            tgButtons, tgButtonTexts, tgButtonRects, options[TG], tgFields, 2, tgUDButtons );
       // fixed weighted sum.
    Rectangle[] tgFixLabelRects = { new Rectangle(24, 138, 40, 17), new Rectangle(114, 138, 44, 17),
                                    new Rectangle(204, 138, 40, 17), new Rectangle(294, 138, 40, 17),
                                    new Rectangle(24, 158, 240, 17) };
    String[] tgFixFieldTexts = {
             String.valueOf(options[TG].getOptionItem(0).getParameter(0).getDoubleValue()),
             String.valueOf(options[TG].getOptionItem(0).getParameter(1).getDoubleValue()),
             String.valueOf(options[TG].getOptionItem(0).getParameter(2).getDoubleValue()),
             String.valueOf(options[TG].getOptionItem(0).getParameter(3).getDoubleValue()),
             String.valueOf(options[TG].getOptionItem(0).getParameter(4).getDoubleValue())
          };
    Rectangle[] tgFixFieldRects = { new Rectangle(64, 138, 32, 17), new Rectangle(158, 138, 32, 17),
                                    new Rectangle(244, 138, 32, 17), new Rectangle(334, 138, 32, 17),
                                    new Rectangle(264, 158, 32, 17) };
    initFieldBlock( combPanels[2], tgFixLabels, wsFixLabelTexts, tgFixLabelRects,
                    tgFields[0], tgFixFieldTexts, tgFixFieldRects, options[TG], 0 );
      // variable weighted sum.
    Rectangle[] tgVarLabelRects = { new Rectangle(24, 48, 40, 17), new Rectangle(114, 48, 44, 17),
                                    new Rectangle(204, 48, 40, 17), new Rectangle(294, 48, 40, 17),
                                    new Rectangle(24,  68, 40, 17), new Rectangle(114, 68, 40, 17),
                                    new Rectangle(24, 88, 240, 17)
                                  };
    String[] tgVarFieldTexts = {
                String.valueOf(options[TG].getOptionItem(1).getParameter(0).getDoubleValue()),
                String.valueOf(options[TG].getOptionItem(1).getParameter(1).getDoubleValue()),
                String.valueOf(options[TG].getOptionItem(1).getParameter(2).getDoubleValue()),
                String.valueOf(options[TG].getOptionItem(1).getParameter(3).getDoubleValue()),
                String.valueOf(options[TG].getOptionItem(1).getParameter(4).getDoubleValue()),
                String.valueOf(options[TG].getOptionItem(1).getParameter(5).getDoubleValue()),
                String.valueOf(options[BU].getOptionItem(1).getParameter(6).getDoubleValue())
             };
    Rectangle[] tgVarFieldRects = { new Rectangle(64, 48, 32, 17), new Rectangle(158, 48, 32, 17),
                                    new Rectangle(244, 48, 32, 17), new Rectangle(334, 48, 32, 17),
                                    new Rectangle(64,  68, 32, 17), new Rectangle(158, 68, 32, 17),
                                    new Rectangle(264, 88, 32, 17)
                                  };
    initFieldBlock( combPanels[2], tgVarLabels, wsVarLabelTexts, tgVarLabelRects,
                    tgFields[1], tgVarFieldTexts, tgVarFieldRects, options[TG], 1 );
    // user defined
    Rectangle[] tgUDRects = { new Rectangle(24, 208, 160, 17), new Rectangle(24, 228, 160, 17) };
    String[] tgUDTexts = { TaskClarion.getSample( TaskClarion.ACS, TaskClarion.TDG ),
                         TaskClarion.getExplanation( TaskClarion.ACS, TaskClarion.TDG ) };
    initUserDefinedBlock( combPanels[2], tgUDButtons, tgUDRects, tgUDTexts );
    combCompOns[2] = new boolean[combPanels[2].getComponentCount()];
    // integration of outcomes of the 2 levels.
    initPanel( outcomePanel, BORDER, null, new Rectangle(10, locations[type][2], 440, 46) );
    String[] outcomeTexts = { "stochastic selection", "bottom-up verification", "top-down guidance" };
    Rectangle[] outcomeRects = { new Rectangle(8, 24, 140, 17), new Rectangle(148, 24, 150, 17),
                                 new Rectangle(298, 24, 140, 17)};
    initButtonBlock( outcomePanel, outcomeLabel, "Integeration of outcomes of the 2 levels",
        new Rectangle(4,4,240,17), outcomeButtons, outcomeTexts, outcomeRects,
        options[COMB], combPanels, combCompOns );
    outcomeCompOns = new boolean[outcomePanel.getComponentCount()];
    // cross-level integration.
    initPanel( crossPanel, BORDER, null, new Rectangle(10, locations[type][1], 440, 46) );
    String[] crossTexts = { "not by MCS", "by the MCS" };
    Rectangle[] crossRects = { new Rectangle(8, 24, 90, 17), new Rectangle(146, 24, 90, 17) };
    initButtonBlock(crossPanel, deterLabel, "cross-level integration", new Rectangle(4,4,340,17),
                    crossButtons, crossTexts, crossRects, options[DETER], 0, outcomePanel,
                    outcomeCompOns, options[COMB], combPanels, combCompOns );
    super.initPanelStatus( options[DETER], 0, outcomePanel, outcomeCompOns, options[COMB], combPanels,
                           combCompOns );
    // return buttons.
    initReturnButton( returnButton, "RETURN",new Rectangle(750, 545, 110, 29), 0, 2 );
    // add components.
    compPanel.add( compLabel );
    compPanel.add( irlNumLabel );
    compPanel.add( frNumLabel );
    compPanel.add( irlNumField );
    compPanel.add( frNumField );
    this.getContentPane().add( caption, null );
    this.getContentPane().add( compPanel, null );
    this.getContentPane().add( eligPanel, null );
    this.getContentPane().add( crossPanel, null );
    this.getContentPane().add( combPanels[0], null );
    this.getContentPane().add( combPanels[1], null );
    this.getContentPane().add( combPanels[2], null );
    this.getContentPane().add( outcomePanel, null );
    this.getContentPane().add( returnButton, null );
  }

  /** update GUI components status resulting from upper level configuration. */
  public void update()
  {
    if( netType != Global.EX_TYPE )
        return;
    // eligibility
    if( eligOption.getUserChoice() == 1 ){
        if( eligPanel.isEnabled() )
            return;
        eligPanel.setEnabled( true );
        for( int j = 0; j < eligCompOns.length; j++ )
             eligPanel.getComponent(j).setEnabled( eligCompOns[j] );
    }
    else{
        if( !eligPanel.isEnabled() )
            return;
        for( int j = 0; j < eligCompOns.length; j++ )
             eligCompOns[ j ] = eligPanel.getComponent(j).isEnabled();
        eligPanel.setEnabled( false );
        for( int j = 0; j < eligPanel.getComponentCount(); j++ )
             eligPanel.getComponent(j).setEnabled( false );
    }
  }
}
