package clarion.options.transfer.subsys;
import java.io.*;
import clarion.system.*;
import clarion.tools.*;
import clarion.options.*;
import clarion.options.transfer.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to implement the process of transferring Dimension information
*  configuration data between GUI and <tt>Global</tt> object or persistent
* storage.
 * <li> important methods <br>
 * <ol>
 * <li> public void load(): implement the process of loading options from persistent
 * storage.
 * <li> public void save(): implement the process of save the options to persistent
 * storage or to the associated <tt>Global</tt> object.
*  <li> public void update(): update the values of Global variables or options.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model. <br>
 * This class is part of data transfer subsystem and is not related directly to
 * CLARION model.
 * <li> Usage of this class. <br>
 * It extends the base <tt>Transfer</t>> class and is instantiated in the class
 * <em>TransferManager</em>.
 * </ul>
 * @author Xi Zhang
 *
 **/

public class DimsTransfer extends Transfer {

  /** constructor. instantiated with the associated <tt>TransferManager</tt> object
   * and <tt>Global</tt> object. */
  public DimsTransfer( TransferManager manager, Global global )
  {
     super( manager, global );
  }

  /** the parameter 'type' starts from Global.FT_INPUT =1, not Global.FT_SYS = 0.
   *  uesrOptions stores info starting from INPUT feature(index = 0),
   *  Get the number of features in the specified dimension type.
   *  @param type the dimension type.
   *  @return the number of features in the specified dimension type. */
  public int getFeatureNum( int type )
  {
     return userOptions[0][0][0][0][0][0].getOptionItem(0).getParameter(type-1).getIntValue();
  }

  /** the parameter 'type' starts from Global.FT_INPUT =1, not Global.FT_SYS = 0.
   *  uesrOptions stores info starting from INPUT feature(index = 0),
   *  Get the name of a feature in the specified type and order in that type.
      *  @param type the dimension type.
      *  @param idx the order in that type.
      *  @return the name of a feature. */
  public String getFeatureName( int type, int idx )
  {
     return userOptions[0][0][0][0][0][1].getOptionItem(0).getParameter(type-1).getStringValues()[idx];
  }

  /** the parameter 'type' starts from Global.FT_INPUT =1, not Global.FT_SYS = 0.
   *  uesrOptions stores info starting from INPUT feature(index = 0),
   *  Get the order of a feature in a type with the particular name. return -1 if
   *  the feature is not in the type.
   *  @param type the dimension type.
   *  @param name the feature name.
   *  @return the order of a feature in a type. */
  public int searchFeatureName( int type, String name )
  {
     String[] names = userOptions[0][0][0][0][0][1].getOptionItem(0).getParameter(type-1).getStringValues();
     for( int i = 0; i < names.length; i++ )
          if( names[i].equals( name ) )
              return i;
     return -1;
  }

  /** implement the process of loading options from persistent storage. */
  public void load()
  {
     loadedFileName = "dims.txt";
     String path = global.getSettingDir() + "\\DEFAULT\\subjectGroup0\\dims\\";
     sysOptions = new Option[1][1][1][1][1][];
     sysOptions[0][0][0][0][0] = loadOptions( path );
     //path = global.getSettingDir() + "\\" + global.getTaskName()
     //              + "\\" + global.getTaskSettingName() + "\\dims\\";
     subsysPath = this.searchFile( "dims" );
     userOptions = new Option[1][1][1][1][1][];
     userOptions[0][0][0][0][0] = loadOptions( subsysPath );
  }

  /** implement the process of save the options to persistent storage or to the
   * associated <tt>Global</tt> object. */
  public void save()
  {
     if( saveCode == SAVE_TO_GLOBAL ){
         initGlobalFeatures();
         return;
     }
     // create DIM setting directory.
     String path = global.getSettingDir() + "\\" + global.getTaskName()
                   + "\\" + global.getTaskSettingName() + "\\dims";
     new File( path ).mkdir();
     savedFileName = "\\dims.txt";
     try{
        new File( path + savedFileName ).createNewFile();
     }
     catch( Exception e ){
        System.out.println( e );
     }
     // save DIM setting.
     saveOptions( path, userOptions[0][0][0][0][0] );
  }

  /** update the values of Global variables or options.  */
  public void update()
  {
     // update the feature information.
     // WM flags.
     int featLength = global.getWmFlagNum();
     global.setFeatureLength(Global.FT_SYS, Global.FT_SYS_WM_FLAG, featLength );
     String[] valNames = new String[ featLength ];
     for( int i = 0; i < featLength; i++ )
          valNames[i] = "WM FLAG " + i;
     global.setFeatureValueNames(Global.FT_SYS, Global.FT_SYS_WM_FLAG, valNames );
     // WM ACT PARAMETER.
     featLength = global.getWmSize();
     global.setFeatureLength( Global.FT_SYS, Global.FT_SYS_WM_ACT_PARA, featLength );
     valNames = new String[ featLength ];
     for( int i = 0; i < featLength; i++ )
          valNames[i] = "WM SLOT " + i;
     global.setFeatureValueNames( Global.FT_SYS, Global.FT_SYS_WM_ACT_PARA, valNames );
     // nacs control: FT_NACS_AMNS
     featLength = global.getAmnNum();
     global.setFeatureLength( Global.FT_SYS, Global.FT_SYS_NACS_AMNS, featLength );
     valNames = new String[ featLength ];
     for( int i = 0; i < featLength; i++ )
          valNames[i] = "AMN NET " + i;
     global.setFeatureValueNames( Global.FT_SYS, Global.FT_SYS_NACS_AMNS, valNames );
     // nacs control: FT_NACS_AMNS
     featLength = global.getGksMaxChunkNum();
     global.setFeatureLength( Global.FT_SYS, Global.FT_SYS_NACS_CHKS, featLength );
     valNames = new String[ featLength ];
     for( int i = 0; i < featLength; i++ )
          valNames[i] = "NACS CHUNK " + i;
     global.setFeatureValueNames( Global.FT_SYS, Global.FT_SYS_NACS_CHKS, valNames );
     // fill output dim mutiple-valued information.
     // fill in EX output dim mutiple-valued information.
     short[] featProperty = global.getFeatureMultiValueProperty( Global.FT_OUTPUT );
     global.setOutputMultiValueProperty( Global.EX_TYPE, featProperty );
     // fill in GS output dim mutiple-valued information.
     featProperty = global.getFeatureMultiValueProperty( Global.FT_GS );
     short[] outputProperty = new short[featProperty.length + 1];
     outputProperty[0] = global.getFeatureMultiValueProperty(Global.FT_SYS)[Global.FT_SYS_GS_ACT];
     for( int i = 0; i  < featProperty.length; i++ )
          outputProperty[i+1] = featProperty[i];
     global.setOutputMultiValueProperty( Global.GS_TYPE, outputProperty );
     // fill in the output information.
     // EX output.
     short[][] acsPhysicOutputDimVals = new short[Global.ACS_OUTPUT_TYPE_NUM-1][];
     int dimNum = global.getFeatureNums()[Global.FT_OUTPUT];
     acsPhysicOutputDimVals[Global.EX_TYPE] = new short[ dimNum ];
     for( int i = 0; i <  dimNum; i++ )
          if( global.getOutputMultiValueProperty( Global.EX_TYPE )[i] == Global.SINGLE_ACTIVE_VALUED )
              acsPhysicOutputDimVals[Global.EX_TYPE][i] = (short)(global.getFeatureLength(Global.FT_OUTPUT, i ) + 2);
          else
              acsPhysicOutputDimVals[Global.EX_TYPE][i] = (short)(2 * global.getFeatureLength(Global.FT_OUTPUT, i ) + 2);
     // WM output.
     acsPhysicOutputDimVals[Global.WM_TYPE] = new short[ Global.ACT_DIM_NUM_WM ];
     acsPhysicOutputDimVals[Global.WM_TYPE][0] = (short)(global.getFeatureLength( Global.FT_SYS, Global.FT_SYS_WM_ACT ) + 2);
     acsPhysicOutputDimVals[Global.WM_TYPE][1] = (short)(2 * global.getFeatureLength( Global.FT_SYS, Global.FT_SYS_WM_ACT_PARA ) + 2);
     // GS output.
     dimNum = global.getFeatureNums()[Global.FT_GS] + 1;
     acsPhysicOutputDimVals[Global.GS_TYPE] = new short[ dimNum ];
     acsPhysicOutputDimVals[Global.GS_TYPE][0] = (short)(global.getFeatureLength( Global.FT_SYS, Global.FT_SYS_GS_ACT ) + 2);
     for( int i = 1; i < dimNum; i++ )
          if( global.getOutputMultiValueProperty( Global.GS_TYPE )[i] == Global.SINGLE_ACTIVE_VALUED )
              acsPhysicOutputDimVals[Global.GS_TYPE][i] = (short)(global.getFeatureLength(Global.FT_GS, i-1 ) + 2);
          else
              acsPhysicOutputDimVals[Global.GS_TYPE][i] = (short)(2 * global.getFeatureLength(Global.FT_GS, i-1 ) + 2);
     global.setAcsPhysicOutputDimVals( acsPhysicOutputDimVals );
     // NACS control output.
     short[] nacsControlOutputDimVals = new short[Global.ACT_DIM_NUM_NACS_CONTROL];
     short[] nacsControlPhysicOutputDimVals = new short[Global.ACT_DIM_NUM_NACS_CONTROL];
     for( int i = 0; i < Global.ACT_DIM_NUM_NACS_CONTROL; i++ ){
          nacsControlOutputDimVals[i] = (short)(global.getFeatureLength( Global.FT_SYS, Global.NACS_CONTROL_DIM_START + i ));
          if( global.getOutputMultiValueProperty( Global.CONTROL_TYPE )[i] == Global.SINGLE_ACTIVE_VALUED )
              nacsControlPhysicOutputDimVals[i] = (short)(nacsControlOutputDimVals[i] + 2);
          else
              nacsControlPhysicOutputDimVals[i] = (short)(2 * nacsControlOutputDimVals[i] + 2);
     }
     global.setNacsControlOutputDimVals( nacsControlOutputDimVals );
     global.setNacsControlPhysicOutputDimVals( nacsControlPhysicOutputDimVals );
     // ACS output numbers of different action type.
     short[] acsPhysicOutputNums = new short[Global.ACS_OUTPUT_TYPE_NUM];
     acsPhysicOutputNums[Global.EX_TYPE] = (short)CommonMethods.getSum( global.getAcsPhysicOutputDimVals(Global.EX_TYPE) );
     acsPhysicOutputNums[Global.WM_TYPE] = (short)CommonMethods.getSum( global.getAcsPhysicOutputDimVals(Global.WM_TYPE) );
     acsPhysicOutputNums[Global.GS_TYPE] = (short)CommonMethods.getSum( global.getAcsPhysicOutputDimVals(Global.GS_TYPE) );
     acsPhysicOutputNums[Global.CONTROL_TYPE] = (short)CommonMethods.getSum( global.getNacsControlPhysicOutputDimVals() );
     global.setAcsPhysicOutputNums( acsPhysicOutputNums );
  }

  /**
   * initialize the variables storing the information of feature definitions:
   * SYSTEM-defined, EX-input, EX-output, GS (goal parameter dims + one goal dim).
   * NOTE:
   * 1. User only needs specify all of the Goal Structure parameter dimensions.
   *    The Goal-ID dimension will be automatically added into the end of the
   *    above list of GS parameter dimensions.
   * 2. When initializing SYSTEM-defined features, the following four features
   *    need the info of other types : EX-input, EX-output.
   *    FT_SYS_NACS_IN_IN, FT_SYS_NACS_IN_OUT, FT_SYS_NACS_OUT_IN, FT_SYS_NACS_OUT_OUT
   * 3. All of the feature-related variables can be initialized in this method except
   *    the following four System-defined features :
   *    Global.FT_SYS_WM_FLAG, Global.FT_SYS_WM_ACT_PARA,
   *    Global.FT_SYS_NACS_AMNS, Global.FT_SYS_NACS_CHKS,
   *    these need info of other options: # of WM flags, WM size, # of AMNs, maximal
   *    # of GKS chunks. The four relevant variables will be initialized in other
   *    methods only after the necessary info is available.  */
  private void initGlobalFeatures()
  {
    final int FEAT_NUMS = 0, FEAT_NAMES = 1, START = 2, V_START = 5;
    Option[] opts = userOptions[0][0][0][0][0];

    // feature type number
    int typeNum = Global.FEAT_TYPE_NUM;
    //global.setFeatureTypeNum( typeNum );
    // feature numbers
    short[] featNums = new short[ typeNum ];
    featNums[Global.FT_SYS] = global.getFeatureNums()[Global.FT_SYS];
    for( int i = 1; i < typeNum; i++ ){
         short num = (short)(opts[FEAT_NUMS].getOptionItem(0).getParameter(i-1).getIntValue());
         if(i == Global.FT_GS && manager.getGsOption() != Global.GS_NON) // count the goal-ID dim.
            num++;
         featNums[i] = num;
         global.setFeatureNum( i, featNums[i] );
    }
    // feature names.
    String[][] featNames = new String[ typeNum ][];
    featNames[Global.FT_SYS] = global.getFeatureNames( Global.FT_SYS );
    for( int i = 1; i < typeNum; i++ ){
         featNames[i] = new String[ featNums[i] ];
         for( int j = 0; j < featNums[i]; j++ )
              if( i != Global.FT_GS || j < featNums[i]-1 )
                  featNames[i][j] = opts[FEAT_NAMES].getOptionItem(0).getParameter(i-1).getStringValues()[j];
              else   // add the name of Goal-ID dim
                  featNames[i][j] = Global.GS_ID_DIM_NAME;
         global.setFeatureNames( i, featNames[i] );
    }
    // feature lengths.
    short[][] featLengths = new short[ typeNum ][];
    featLengths[Global.FT_SYS] = global.getFeatureLengths( Global.FT_SYS );
    featLengths[Global.FT_SYS][Global.FT_SYS_NACS_IN_IN] = featNums[Global.FT_INPUT];
    featLengths[Global.FT_SYS][Global.FT_SYS_NACS_IN_OUT] = featNums[Global.FT_OUTPUT];
    featLengths[Global.FT_SYS][Global.FT_SYS_NACS_OUT_IN] = featNums[Global.FT_INPUT];
    featLengths[Global.FT_SYS][Global.FT_SYS_NACS_OUT_OUT] = featNums[Global.FT_OUTPUT];
    for( int i = 1; i < typeNum; i++ ){
         featLengths[i] = new short[ featNums[i] ];
         for( int j = 0; j < featNums[i]; j++ ){
              if( i != Global.FT_GS || j < featNums[i] - 1 )
                  featLengths[i][j] = (short)(opts[START + i - 1].getOptionItem(0).getParameter(0).getIntValues()[j]);
              else    // add the length of Goal-ID dim
                  featLengths[i][j] = (short)(featNums[i] - 1);
         }
    }
    global.setFeatureLengths( featLengths );
    // feature value property.
    short[][] featProperty = new short[ typeNum ][];
    for( int i = 1; i < typeNum; i++ ){
         featProperty[i] = new short[ featNums[i] ];
         for( int j = 0; j < featNums[i]; j++ ){
              if( i != Global.FT_GS || j < featNums[i] - 1 )
                  featProperty[i][j] = (short)(opts[START + i - 1].getOptionItem(0).getParameter(1).getIntValues()[j]);
              else    // add the property of Goal-ID dim
                  featProperty[i][j] = (short)Global.SINGLE_ACTIVE_VALUED;
         }
         global.setFeatureMultiValueProperty( i, featProperty[i] );
    }
    // feature value names.
    String[][][] featValNames = new String[typeNum][][];
    featValNames[Global.FT_SYS] = global.getFeatureValueNames( Global.FT_SYS );
    for( int j = 0; j < featNums[Global.FT_SYS]; j++ ){
         switch( j ){
             case Global.FT_SYS_NACS_IN_IN:
             case Global.FT_SYS_NACS_OUT_IN:
                  int valNum = featLengths[Global.FT_SYS][j];
                  featValNames[Global.FT_SYS][j] = new String[ valNum ];
                  for( int k = 0; k < valNum; k++ )
                       featValNames[Global.FT_SYS][j][k] = "external input dim " + k;
                  break;
             case Global.FT_SYS_NACS_IN_OUT:
             case Global.FT_SYS_NACS_OUT_OUT:
                  valNum = featLengths[Global.FT_SYS][j];
                  featValNames[Global.FT_SYS][j] = new String[ valNum ];
                  for( int k = 0; k < valNum; k++ )
                       featValNames[Global.FT_SYS][j][k] = "external output dim " + k;
                  break;
         }
    }
    for( int i = 1; i < typeNum; i++ ){
         featValNames[i] = new String[ featNums[i] ][];
         for( int j = 0; j < featNums[i]; j++ ){
              featValNames[i][j] = new String[ featLengths[i][j] ];
              for( int k = 0; k < featLengths[i][j]; k++ ){
                   if( i != Global.FT_GS || j < featNums[i] - 1 ){

                          //System.out.println( i + " "  + " " + j + " " + k );


                       featValNames[i][j][k] = opts[V_START+i-1].getOptionItem(0).getParameter(j).getStringValues()[k];
                  } else  // add the value names of Goal-ID dim
                       featValNames[i][j][k] = Global.GS_ID_DIM_NAME + " value " + k;
              }
         }
    }
    global.setFeatureValueNames( featValNames );
  }

}
