package clarion.options.transfer.subsys;
import java.io.*;
import clarion.system.*;
import clarion.options.*;
import clarion.options.transfer.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to implement the process of transferring Output Display configuration
*  data between GUI and <tt>Global</tt> object or persistent storage.
 * <li> important methods <br>
 * <ol>
 * <li> public void load(): implement the process of loading options from persistent
 * storage.
 * <li> public void save(): implement the process of save the options to persistent
 * storage or to the associated <tt>Global</tt> object.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model. <br>
 * This class is part of data transfer subsystem and is not related directly to
 * CLARION model.
 * <li> Usage of this class. <br>
 * It extends the base <tt>Transfer</t>> class and is instantiated in the class
 * <em>TransferManager</em>.
 * </ul>
 * @author Xi Zhang
 *
 **/

public class InterTransfer extends Transfer {

  /** constructor. instantiated with the associated <tt>TransferManager</tt> object
   * and <tt>Global</tt> object. */
  public InterTransfer( TransferManager manager, Global global )
  {
     super( manager, global );
  }

  /** implement the process of loading options from persistent storage. */
  public void load()
  {
     loadedFileName = "display.txt";
     String path = global.getSettingDir() + "\\DEFAULT\\subjectGroup0\\display\\";
     sysOptions = new Option[1][1][1][1][1][];
     sysOptions[0][0][0][0][0] = loadOptions( path );
     //path = global.getSettingDir() + "\\" + global.getTaskName()
     //             + "\\" + global.getTaskSettingName() + "\\display\\";
     subsysPath = this.searchFile( "display" );
     userOptions = new Option[1][1][1][1][1][];
     userOptions[0][0][0][0][0] = loadOptions( subsysPath );
  }

  /** implement the process of save the options to persistent storage or to the
   * associated <tt>Global</tt> object. */
  public void save()
  {
     if( saveCode == SAVE_TO_GLOBAL ){
         initGlobalDisplay();
         return;
     }
     // create Display setting directory.
     String path = global.getSettingDir() + "\\" + global.getTaskName()
                   + "\\" + global.getTaskSettingName() + "\\display";
     new File( path ).mkdir();
     savedFileName = "\\display.txt";
     try{
        new File( path + savedFileName ).createNewFile();
     }
     catch( Exception e ){
        System.out.println( e );
     }
     // save task-specific setting.
     saveOptions( path, userOptions[0][0][0][0][0] );
  }


  /****************** init Global display **********************/
  /** initialize the variables relevant to Output Display during task running in the
   * associated <tt>Global</tt> object. */
  private void initGlobalDisplay()
  {
     final int DISP = 0, FREQ = 1;
     Option[] opts = userOptions[0][0][0][0][0];
     global.setRuleDisplayMode( opts[DISP].getUserChoice() );
     switch( opts[FREQ].getUserChoice() ){
       case 0:
             global.setRuleDisplayFreq(Integer.MAX_VALUE);
             break;
       case 1:
             global.setRuleDisplayFreq(1);
             break;
       case 2:
             global.setRuleDisplayFreq(opts[FREQ].getOptionItem(2).getParameter(0).getIntValue());
             break;
     }
  }

}
