/**
 * XOR Task
 * This class defines the behavior of an agent (subject) in the task.
 */
package tasksimulators.newsimu.XOR;
import clarion.common.*;
import clarion.tools.*;
import clarion.nacs.*;
import clarion.system.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to simulate one subject in XOR task. It extends the
*  <tt>TaskAgent</tt> class.
 * <li> important methods <br>
 * <ol>
 *  <li> public void run() : implements the overall process of a subject during
 *  the task experiment. It is assumed to be called by the <tt>TaskXOR</tt> class
 *  to implement what a subject have done during the experiment.
 *  <li> public void performAction( int netIdx, short[][][] action ) : perform
 *  the selected action from a particular ACS network.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model. <br>
 * This class implemnts the Process Control task using CLARION and part of
 * XOR task simulator.
 * <li> Usage of this class. <br>
 *  Routines in this class will be called by the system when the XOR task is loaded
 *  for simulation.
 * </ul>
 *
 **/

public class AgentXOR extends TaskAgent{

   // used to store the agent's last decision, or output
   private int lastResult;

   // you must override the default constructor even if it's not used
   public AgentXOR()
   {
   }

   // Constructor
   public AgentXOR( Global g )
   {
      // initialize the agent using a "global" (configuration) object
      super( g );
      reinforcements = new double[1];
      // initialize the input that the agent will see
      curSensoryInput = new Feature[ 2 ];
   }

   // Used to set the input the agent will see at each learning step.
   // curSensoryInput will automatically be examined by Clarion.
   public void setSensoryInput(short[] inputs)
   {
      // Each input dimension is encoded as an array of values
      short[] values1 = { inputs[0] };
      short[] values2 = { inputs[1] };
      // Sets the input (see clarion.common/Feature.java for the declaration
      // of the Feature() constructor).  Some notes on this constructor:
      // * Each feature is referenced by its name (1st param)
      // * 'Type' will normally be 0 (2nd param)
      // * 'Order' will be 0 for 1st feature, 1 for second, etc. (3rd param)
      // * 'Length' is the number of values in each dimension (4th param)
      curSensoryInput[0] = new Feature( global.getFeatureNames()[Global.FT_INPUT][0],
                                        (short)Global.FT_INPUT, (short)0, (short)2, values1, global);
      curSensoryInput[1] = new Feature( global.getFeatureNames()[Global.FT_INPUT][1],
                                        (short)Global.FT_INPUT, (short)1, (short)2, values2, global);
   }

   // is the chosen action is an action for controlling NACS?
    public boolean isControlNacsAction( int netIdx, short[] chosenAction )
    {
      return false;
    }

   // Gets the output of current step.  Since there is only one "action"
   // (output) in the XOR task, we set the first value of the array accordingly
    public void getOutput( Object arr )
    {
      ((int[])arr)[0] = lastResult;
    }

   // determine which AMN will be used given current action.
    public int determineAmnNet( int netIdx, short[][][] action ) {
       return 0;
    }

   // send the retrieval results to ACS components.
    public void setRetrievalResults( int netIdx, GKSChunk[] results ) {}

   /** This determines the action to be performed at each step.
       For the XOR task, the only "action" is to set the output correctly.
    * @param netIdx the ACS network index
    * @param action the selected action from the specified ACS network. */
    public void performAction( int netIdx, short[][][] action )
    {
       lastResult = action[Global.ACTION_NORMAL][0][0];
    }

    /** simulates one step in the agent's behavior for this experiment. */
    public void run()
    {
       clarion.training();
    }

}
