/**
 * XOR Task
 * This class defines the task-specific part of the Clarion model's behavior.
 * Each instance of TaskClarionXOR is associated with a single agent.
 */
package tasksimulators.newsimu.XOR;
import clarion.common.*;
import clarion.acs.gs.*;
import clarion.system.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class extends the <tt>TaskClarion</tt> class and implements all
 * of the necessary methods specific to XOR task.
 * <li> summary of the important methods. <br>
 * <ol>
 * <li> public short[][][] getEligibility( int netIdx ) :  eligibility of ACS networks,
 * return the eligible condition of the ACS network.
 * <li> public double getReinforcement() : Returns the reinforcement awarded the agent,
 * based on the previous inputs to the agent, and on the agent's subsequent decision.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model <br>
 * It is a part of the XOR task simulator.
 * <li> Usage of this class. <br>
 * This class is instantiated inside a <tt>TaskAgent</tt> object.
 * </ul>
 * @author Xi Zhang
 */

public class TaskClarionXOR extends TaskClarion{

  public TaskClarionXOR()
  {
  }

  // Determines which of the IDN networks is eligible at each step:
  // * The outermost array stores all the eligibility data.
  // * The middle array (there is only one) indicates that there's only one IDN.
  // * The inner arrays indicate, respectively,
  //   - That for each possible input dimension (0 or 1)
  //   - Both values of the first dimension are eligible (0 and 1)
  //   - And both values of the second dimension are eligible (0 and 1)
  public short[][][] getEligibility( int netIdx )
  {
    short[][][] eligCondition = {
                   {
                      {0,1},
                      {1,1},
                      {1,1}
                   },
                };
     return eligCondition;
  }

  /** Returns the reinforcement awarded the agent, based on the previous inputs
   to the agent, and on the agent's subsequent decision. */
  public double getReinforcement( int netIdx, Feature[] input, Chunk[] wmChunks, GoalChunk goal,
                        short[][][] action, Feature[] newInput, Chunk[] newWMChunks, GoalChunk newGoal )
  {
     // Retrieve the most recent inputs to the agent
     short val1 = input[0].getValues()[0];
     short val2 = input[1].getValues()[0];

     // Determine if the agent was right or wrong, and give a corresponding
     // reward of 1 or 0.
     if( (val1 == 1 && val2 == 1) || (val1 == 0 && val2 == 0) ){
       if(action[Global.ACTION_NORMAL][0][0] == 0)
         return 1;
       else
         return 0;
     }

     else if( (val1 == 0 && val2 == 1) || (val1 == 1 && val2 == 0) ){
       if(action[Global.ACTION_NORMAL][0][0] == 1)
         return 1;
       else
         return 0;
     }

     return 0; // should never happen
  }
}
